module Tcl where

import Data.Char(isSpace)
import Prelude hiding (Word)
import System.Exit(die)
import Control.Exception(catch)
import Text.ParserCombinators.Parsec hiding (State)

import qualified Data.Map as M
import InteractiveEval (Term(val))
import THNames (varBangTKey)

type Script = [Cmd]
type Cmd    = [Word]
data Word   = Word { str :: String }
            | Quotes String
            | Braces String
            | Grp3 Script
            deriving (Eq, Show)

script :: GenParser Char a Script
script = do s<- sepBy cmd newLine; mspc; return s

newLine :: GenParser Char a Char
newLine = oneOf ";\n"

cmd :: GenParser Char a Cmd
cmd = do mspc; sepEndBy word mspc

mspc :: GenParser Char a ()
mspc = do many (char ' '); return () 

word :: GenParser Char a Word
word = group1 <|> group2 <|> group3 <|> chars

chars :: GenParser Char a Word
chars = do str <- many1 (satisfy isTclChar)
           return $ Word str

isTclChar :: Char-> Bool
isTclChar c = not (isSpace c) && c `notElem` ";[]{}\"" 

group1 :: GenParser Char a Word
group1 = do str <- between (char '{') (char '}') (many (noneOf "}"))
            return $ Braces str

group2 :: GenParser Char a Word
group2 = do str <- between (char '"') (char '"') (many (noneOf "\""))
            return $ Quotes str

group3 :: GenParser Char a Word
group3 = do scr <- between (char '[') (char ']') script
            return $ Grp3 scr


group :: String-> Either ParseError Script
group = parse script "?"

loadSource :: FilePath-> IO Script
loadSource fp = (do
   src<- readFile fp
   case parse script fp src of
     Right s -> return s
     Left e -> die $ "ERROR: "++ show e)
  `catch` (\e -> die $ "ERROR: "++ show (e :: IOError))


type State = M.Map String String
